@@ -12,25 +12,16 @@
 # WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 # License for the specific language governing permissions and limitations
 # under the License.
-
 from oslo_config import cfg
 
 # Register options for the service
 OPTS = [
     cfg.StrOpt('paste_config',
-               default="api_paste.ini",
+               default='api-paste.ini',
                help="Configuration file for WSGI definition of API."),
-    cfg.IntOpt('workers', default=1,
-               min=1,
-               help='Number of workers for aodh API server.'),
-    cfg.BoolOpt('pecan_debug',
-                default=False,
-                help='Toggle Pecan Debug Middleware.'),
-    cfg.BoolOpt('enable_combination_alarms',
-                default=False,
-                help="Enable deprecated combination alarms.",
-                deprecated_for_removal=True,
-                deprecated_reason="Combination alarms are deprecated. "
-                "This option and combination alarms will be "
-                "removed in Aodh 5.0."),
+    cfg.StrOpt(
+        'auth_mode',
+        default="keystone",
+        choices=('keystone', 'noauth'),
+        help="Authentication mode to use."),
 ]
