@@ -59,10 +59,12 @@ class AlarmGnocchiThresholdRule(base.AlarmRule):
     aggregation_method = wsme.wsattr(wtypes.text, mandatory=True)
     "The aggregation_method to compare to the threshold"
 
-    evaluation_periods = wsme.wsattr(wtypes.IntegerType(minimum=1), default=1)
+    evaluation_periods = wsme.wsattr(wtypes.IntegerType(
+        minimum=1, maximum=100), default=1)
     "The number of historical periods to evaluate the threshold"
 
-    granularity = wsme.wsattr(wtypes.IntegerType(minimum=1), default=60)
+    granularity = wsme.wsattr(wtypes.IntegerType(
+        minimum=1, maximum=3600 * 24 * 365), default=60)
     "The time range in seconds over which query"
 
     cache = cachetools.TTLCache(maxsize=1, ttl=3600)
@@ -74,7 +76,7 @@ class AlarmGnocchiThresholdRule(base.AlarmRule):
         aggregation_method = alarm_rule.aggregation_method
         if aggregation_method not in cls._get_aggregation_methods():
             raise base.ClientSideError(
-                'aggregation_method should be in %s not %s' % (
+                'aggregation_method should be in {} not {}'.format(
                     cls._get_aggregation_methods(), aggregation_method))
 
     @staticmethod
@@ -162,8 +164,7 @@ class AggregationMetricByResourcesLookupRule(AlarmGnocchiThresholdRule):
 
     @classmethod
     def validate_alarm(cls, alarm):
-        super(AggregationMetricByResourcesLookupRule,
-              cls).validate_alarm(alarm)
+        super().validate_alarm(alarm)
 
         rule = alarm.gnocchi_aggregation_by_resources_threshold_rule
 
