@@ -20,7 +20,7 @@ from oslo_utils import uuidutils
 import sqlalchemy as sa
 from sqlalchemy import Column, String, Index, Boolean, Text, DateTime, Integer
 from sqlalchemy.dialects import mysql
-from sqlalchemy.ext.declarative import declarative_base
+from sqlalchemy.orm import declarative_base
 from sqlalchemy.types import TypeDecorator
 
 
@@ -55,7 +55,7 @@ class TimestampUTC(TypeDecorator):
         return self.impl
 
 
-class AodhBase(object):
+class AodhBase:
     """Base class for Aodh Models."""
     __table_args__ = {'mysql_charset': "utf8",
                       'mysql_engine': "InnoDB"}
@@ -119,7 +119,9 @@ class AlarmChange(Base):
         Index('ix_alarm_history_alarm_id', 'alarm_id'),
     )
     event_id = Column(String(128), primary_key=True)
-    alarm_id = Column(String(128))
+    alarm_id = Column(String(128),
+                      sa.ForeignKey('alarm.alarm_id', ondelete='CASCADE'),
+                      nullable=False)
     on_behalf_of = Column(String(128))
     project_id = Column(String(128))
     user_id = Column(String(128))
@@ -141,3 +143,24 @@ class Quota(Base):
     project_id = Column(String(128), nullable=False)
     resource = Column(String(50), nullable=False)
     limit = Column(Integer, nullable=False)
+
+
+class AlarmCounter(Base):
+    __tablename__ = 'alarm_counter'
+    __table_args__ = (
+        sa.UniqueConstraint('alarm_id', 'project_id', 'state'),
+        Index('ix_%s_alarm_id' % __tablename__,
+              'alarm_id'),
+        Index('ix_%s_state' % __tablename__,
+              'state'),
+        Index('ix_%s_project_id' % __tablename__,
+              'project_id'),
+    )
+
+    id = Column(String(36), primary_key=True, default=uuidutils.generate_uuid)
+    alarm_id = Column(String(128),
+                      sa.ForeignKey('alarm.alarm_id', ondelete='CASCADE'),
+                      nullable=False)
+    project_id = Column(String(128), nullable=False)
+    state = Column(String(128), nullable=False)
+    value = Column(Integer, nullable=False)
