@@ -16,8 +16,8 @@
 """
 
 import datetime
+from unittest import mock
 
-import mock
 from oslo_utils import timeutils
 
 from aodh import storage
@@ -42,7 +42,7 @@ class DBTestBase(tests_db.TestBase):
         return side_effect
 
     def setUp(self):
-        super(DBTestBase, self).setUp()
+        super().setUp()
         patcher = mock.patch.object(timeutils, 'utcnow')
         self.addCleanup(patcher.stop)
         self.mock_utcnow = patcher.start()
@@ -166,13 +166,12 @@ class AlarmTest(AlarmTestBase):
 
     def test_list_by_type(self):
         self.add_some_alarms()
-        alarms = list(self.alarm_conn.get_alarms(alarm_type=ALARM_TYPE))
+        alarms = list(self.alarm_conn.get_alarms(type=ALARM_TYPE))
         self.assertEqual(3, len(alarms))
 
     def test_list_excluded_by_name(self):
         self.add_some_alarms()
-        exclude = {'name': 'yellow-alert'}
-        alarms = list(self.alarm_conn.get_alarms(exclude=exclude))
+        alarms = list(self.alarm_conn.get_alarms(name={'ne': 'yellow-alert'}))
         self.assertEqual(2, len(alarms))
         alarm_names = sorted([a.name for a in alarms])
         self.assertEqual(['orange-alert', 'red-alert'], alarm_names)
@@ -246,9 +245,30 @@ class AlarmTest(AlarmTestBase):
     def test_delete(self):
         self.add_some_alarms()
         victim = list(self.alarm_conn.get_alarms(name='orange-alert'))[0]
+        self.alarm_conn.increment_alarm_counter(
+            victim.alarm_id,
+            victim.project_id,
+            "ok"
+        )
+        self.assertEqual(
+            1,
+            self.alarm_conn.get_alarm_counters(
+                victim.alarm_id,
+                victim.project_id,
+                "ok"
+            )[0].value
+        )
         self.alarm_conn.delete_alarm(victim.alarm_id)
         survivors = list(self.alarm_conn.get_alarms())
         self.assertEqual(2, len(survivors))
+        self.assertEqual(
+            [],
+            self.alarm_conn.get_alarm_counters(
+                victim.alarm_id,
+                victim.project_id,
+                "ok"
+            )
+        )
         for s in survivors:
             self.assertNotEqual(victim.name, s.name)
 
@@ -278,7 +298,7 @@ class AlarmHistoryTest(AlarmTestBase):
 
     def _clear_alarm_history(self, utcnow, ttl, count):
         self.mock_utcnow.return_value = utcnow
-        self.alarm_conn.clear_expired_alarm_history_data(ttl)
+        self.alarm_conn.clear_expired_alarm_history_data(ttl, 100)
         history = list(self.alarm_conn.query_alarm_history())
         self.assertEqual(count, len(history))
 
@@ -318,7 +338,7 @@ class AlarmHistoryTest(AlarmTestBase):
             "on_behalf_of": alarm.project_id,
             "severity": severity,
             "timestamp": datetime.datetime(2014, 4, 7, 7, 34)
-            }
+        }
         self.alarm_conn.record_alarm_change(alarm_change=alarm_change)
         filter_expr = {"=": {"severity": "low"}}
         history = list(self.alarm_conn.query_alarm_history(
@@ -351,7 +371,7 @@ class ComplexAlarmQueryTest(AlarmTestBase):
 
         self.assertEqual(1, len(result))
         for a in result:
-            self.assertIn(a.name, set(["yellow-alert", "red-alert"]))
+            self.assertIn(a.name, {"yellow-alert", "red-alert"})
             self.assertTrue(a.enabled)
 
     def test_filter_with_regexp(self):
@@ -507,3 +527,55 @@ class ComplexAlarmHistoryQueryTest(AlarmTestBase):
                           alarm_models.AlarmChange.RULE_CHANGE,
                           alarm_models.AlarmChange.STATE_TRANSITION],
                          [h.type for h in history])
+
+
+class AlarmCounterTest(AlarmTestBase):
+    def test_get_value_of_empty_counter(self):
+        counter_name = "some_empty_unused_counter"
+        self.assertEqual([], self.alarm_conn.get_alarm_counters(
+            "some_alarm_id",
+            "some_project_id",
+            counter_name))
+
+    def test_counter_increment(self):
+        self.add_some_alarms()
+        alarm = list(self.alarm_conn.get_alarms(name='orange-alert'))[0]
+        counter_name = "counter_for_increment_testing"
+        project_id = alarm.project_id
+        alarm_id = alarm.alarm_id
+
+        self.assertEqual([], self.alarm_conn.get_alarm_counters(
+            alarm_id,
+            project_id,
+            counter_name
+        ))
+
+        for i in range(5):
+            self.alarm_conn.increment_alarm_counter(
+                alarm_id,
+                project_id,
+                counter_name
+            )
+        self.assertEqual(
+            5,
+            self.alarm_conn.get_alarm_counters(
+                alarm_id,
+                project_id,
+                counter_name
+            )[0].value
+        )
+
+        for i in range(3):
+            self.alarm_conn.increment_alarm_counter(
+                alarm_id,
+                project_id,
+                counter_name
+            )
+        self.assertEqual(
+            8,
+            self.alarm_conn.get_alarm_counters(
+                alarm_id,
+                project_id,
+                counter_name
+            )[0].value
+        )
